/*
 * Copyright © 2010 Red Hat, Inc.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package com.redhat.rhevm.api.powershell.resource;

import javax.ws.rs.core.UriInfo;

import com.redhat.rhevm.api.common.resource.AbstractUpdatableResource;
import com.redhat.rhevm.api.common.util.LinkHelper;
import com.redhat.rhevm.api.common.util.ReapedMap;
import com.redhat.rhevm.api.model.BaseResource;
import com.redhat.rhevm.api.model.Link;

public abstract class AbstractPowerShellCollectionResource<R extends BaseResource,
                                                           U extends AbstractUpdatableResource<R>>
    extends InjectableUriProviderBase {

    private ReapedMap<String, U> resources;

    public AbstractPowerShellCollectionResource() {
        resources = new ReapedMap<String, U>();
    }

    protected U getSubResource(String id) {
        synchronized (resources) {
            U ret = resources.get(id);
            if (ret == null) {
                ret = createSubResource(id);
                resources.put(id, ret);
                resources.reapable(id);
            }
            return ret;
        }
    }

    protected void removeSubResource(String id) {
        synchronized (resources) {
            resources.remove(id);
        }
    }

    protected abstract U createSubResource(String id);

    protected R addStatus(UriInfo uriInfo, R model, String taskIds) {
        if (taskIds != null) {
            Link link = new Link();
            link.setRel(CREATION_STATUS);
            link.setHref(LinkHelper.getUriBuilder(uriInfo, model).path(CREATION_STATUS).path(taskIds).build().toString());
            model.getLinks().add(link);
        }
        return model;
    }

}
